=== modified file 'bzrlib/branch.py'
--- bzrlib/branch.py	2007-02-06 02:33:42 +0000
+++ bzrlib/branch.py	2007-02-12 22:36:18 +0000
@@ -1446,7 +1446,7 @@
             if master_branch:
                 master_branch.unlock()
 
-    @needs_write_lock
+    @needs_read_lock
     def push(self, target, overwrite=False, stop_revision=None):
         """Updates branch.push to be bound branch aware."""
         bound_location = target.get_bound_location()

=== modified file 'bzrlib/tests/branch_implementations/test_push.py'
--- bzrlib/tests/branch_implementations/test_push.py	2007-02-06 02:33:42 +0000
+++ bzrlib/tests/branch_implementations/test_push.py	2007-02-12 22:58:48 +0000
@@ -90,6 +90,52 @@
         self.assertRaises(errors.BoundBranchConnectionFailure,
                 other.branch.push, checkout.branch)
 
+    def test_push_uses_read_lock(self):
+        """Push should only need a read lock on the source side."""
+        source = self.make_branch_and_tree('source')
+        target = self.make_branch('target')
+
+        self.build_tree(['source/a'])
+        source.add(['a'])
+        source.commit('a')
+
+        source.branch.lock_read()
+        try:
+            target.lock_write()
+            try:
+                source.branch.push(target, stop_revision=source.last_revision())
+            finally:
+                target.unlock()
+        finally:
+            source.branch.unlock()
+
+    def test_push_within_repository(self):
+        """Push from one branch to another inside the same repository."""
+        try:
+            repo = self.make_repository('repo', shared=True)
+        except (errors.IncompatibleFormat, errors.UninitializableFormat):
+            # This Branch format cannot create shared repositories
+            return
+        # This is a little bit trickier because make_branch_and_tree will not
+        # re-use a shared repository.
+        a_bzrdir = self.make_bzrdir('repo/tree')
+        try:
+            a_branch = self.branch_format.initialize(a_bzrdir)
+        except (errors.UninitializableFormat):
+            # Cannot create these branches
+            return
+        tree = a_branch.bzrdir.create_workingtree()
+        self.build_tree(['repo/tree/a'])
+        tree.add(['a'])
+        tree.commit('a')
+
+        to_bzrdir = self.make_bzrdir('repo/branch')
+        to_branch = self.branch_format.initialize(to_bzrdir)
+        tree.branch.push(to_branch)
+
+        self.assertEqual(tree.branch.last_revision(),
+                         to_branch.last_revision())
+
 
 class TestPushHook(TestCaseWithBranch):
 

